"""
FingerprintSwitcher is like user agent switcher on steroids.

It changes not only user agent, but a lot of browser internals to make BAS actually look like firefox, chrome, safari,
on desktop or on mobile.
"""
